@@ -32,8 +32,9 @@ sub add_default_response {
   for my $route ($self->routes->each) {
     my $op = $self->get([paths => @$route{qw(path method)}]);
     for my $status (@{$params->{status}}) {
-      $op->{responses}{$status}{description} //= $params->{description};
+      next if $self->get(['paths', @$route{qw(path method)}, 'responses', $status]);
       $op->{responses}{$status}{content}{'application/json'} //= {schema => $ref};
+      $op->{responses}{$status}{description} //= $params->{description};
     }
   }
 
@@ -185,7 +186,7 @@ sub _coerce_parameter_style_object {
   my $explode = $param->{explode} // (grep { $style eq $_ } qw(cookie query)) ? 1 : 0;
 
   if ($explode) {
-    return if $style eq 'form';
+    return $self->_coerce_parameter_style_object_form($val, $param) if $style eq 'form';
     state $style_re = {label => qr{\.}, matrix => qr{;}, simple => qr{,}};
     return unless my $re = $style_re->{$style};
     return if $style eq 'matrix' && $val->{value} !~ s/^;//;
@@ -241,6 +242,16 @@ sub _coerce_parameter_style_object_deep {
   return $val->{exists} = 0;
 }
 
+sub _coerce_parameter_style_object_form {
+  my ($self, $val, $param) = @_;
+  return unless my $properties = $param->{schema} && $param->{schema}{properties};
+
+  for my $k (keys %{$val->{value}}) {
+    next unless my $type = $properties->{$k} && $properties->{$k}{type};
+    $val->{value}{$k} = [$val->{value}{$k}] if $type eq 'array' and ref $val->{value}{$k} ne 'ARRAY';
+  }
+}
+
 sub _get_parameter_value {
   my ($self, $param, $get) = @_;
   my $schema_type = schema_type $param->{schema};
@@ -345,7 +356,7 @@ sub _validate_type_object_request {
 
   my (@errors, %ro);
   for my $name (keys %{$schema->{properties} || {}}) {
-    next unless $schema->{properties}{$name}{readOnly};
+    next unless $self->_get(['readOnly'], {path => [], schema => $schema->{properties}{$name}});
     push @errors, E [@$path, $name], "Read-only." if exists $data->{$name};
     $ro{$name} = 1;
   }
@@ -366,7 +377,7 @@ sub _validate_type_object_response {
 
   my (@errors, %rw);
   for my $name (keys %{$schema->{properties} || {}}) {
-    next unless $schema->{properties}{$name}{writeOnly};
+    next unless $self->_get(['writeOnly'], {path => [], schema => $schema->{properties}{$name}});
     push @errors, E [@$path, $name], "Write-only." if exists $data->{$name};
     $rw{$name} = 1;
   }
